"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Hint = void 0;
var react_1 = __importStar(require("react"));
var utils_1 = require("./utils");
exports.Hint = function (props) {
    var child = react_1.default.Children.only(props.children);
    var options = props.options, disableHint = props.disableHint, allowTabFill = props.allowTabFill;
    var childProps = child.props;
    var mainInputRef = react_1.useRef(null);
    var hintWrapperRef = react_1.useRef(null);
    var hintRef = react_1.useRef(null);
    var _a = react_1.useState(''), text = _a[0], setText = _a[1];
    var _b = react_1.useState(''), hint = _b[0], setHint = _b[1];
    var _c = react_1.useState(), changeEvent = _c[0], setChangeEvent = _c[1];
    react_1.useEffect(function () {
        if (disableHint) {
            return;
        }
        var inputStyle = mainInputRef.current && window.getComputedStyle(mainInputRef.current);
        inputStyle && styleHint(hintWrapperRef, hintRef, inputStyle);
    });
    var getHint = function (text) {
        if (!text || text === '') {
            return '';
        }
        var match = options
            .filter(function (x) { return x.toLowerCase() !== text.toLowerCase() && x.toLowerCase().startsWith(text.toLowerCase()); })
            .sort()[0];
        return match
            ? match.slice(text.length)
            : '';
    };
    var setAvailableHint = function () {
        if (hint !== '') {
            if (changeEvent) {
                changeEvent.target.value = text + hint;
                childProps.onChange && childProps.onChange(changeEvent);
                setHint('');
            }
        }
    };
    var styleHint = function (hintWrapperRef, hintRef, inputStyle) {
        var _a, _b;
        if ((_a = hintWrapperRef === null || hintWrapperRef === void 0 ? void 0 : hintWrapperRef.current) === null || _a === void 0 ? void 0 : _a.style) {
            hintWrapperRef.current.style.fontFamily = inputStyle.fontFamily;
            hintWrapperRef.current.style.fontSize = inputStyle.fontSize;
            hintWrapperRef.current.style.width = inputStyle.width;
            hintWrapperRef.current.style.height = inputStyle.height;
            hintWrapperRef.current.style.lineHeight = inputStyle.lineHeight;
            hintWrapperRef.current.style.boxSizing = inputStyle.boxSizing;
            hintWrapperRef.current.style.margin = utils_1.interpolateStyle(inputStyle, 'margin');
            hintWrapperRef.current.style.padding = utils_1.interpolateStyle(inputStyle, 'padding');
            hintWrapperRef.current.style.borderStyle = utils_1.interpolateStyle(inputStyle, 'border', 'style');
            hintWrapperRef.current.style.borderWidth = utils_1.interpolateStyle(inputStyle, 'border', 'width');
        }
        if ((_b = hintRef === null || hintRef === void 0 ? void 0 : hintRef.current) === null || _b === void 0 ? void 0 : _b.style) {
            hintRef.current.style.fontFamily = inputStyle.fontFamily;
            hintRef.current.style.fontSize = inputStyle.fontSize;
            hintRef.current.style.lineHeight = inputStyle.lineHeight;
        }
    };
    var onChange = function (e) {
        setChangeEvent(e);
        e.persist();
        setText(e.target.value);
        setHint(getHint(e.target.value));
        childProps.onChange && childProps.onChange(e);
    };
    var onFocus = function (e) {
        setHint(getHint(e.target.value));
        childProps.onFocus && childProps.onFocus(e);
    };
    var onBlur = function (e) {
        //Only blur it if the new focus isn't the the hint input
        if ((hintRef === null || hintRef === void 0 ? void 0 : hintRef.current) !== e.relatedTarget) {
            setHint('');
            childProps.onBlur && childProps.onBlur(e);
        }
    };
    var ARROWRIGHT = 'ArrowRight';
    var TAB = 'Tab';
    var onKeyDown = function (e) {
        var caretIsAtTextEnd = (function () {
            // For selectable input types ("text", "search"), only select the hint if
            // it's at the end of the input value. For non-selectable types ("email",
            // "number"), always select the hint.
            var isNonSelectableType = e.currentTarget.selectionEnd == null;
            var caretIsAtTextEnd = isNonSelectableType
                ? true
                : e.currentTarget.selectionEnd === e.currentTarget.value.length;
            return caretIsAtTextEnd;
        })();
        if (caretIsAtTextEnd && e.key === ARROWRIGHT) {
            setAvailableHint();
        }
        else if (caretIsAtTextEnd && allowTabFill && e.key === TAB && hint !== '') {
            e.preventDefault();
            setAvailableHint();
        }
        childProps.onKeyDown && childProps.onKeyDown(e);
    };
    var onHintClick = function (e) {
        var _a;
        var hintCaretPosition = e.currentTarget.selectionEnd || 0;
        // If user clicks the position before the first character of the hint, 
        // move focus to the end of the mainInput text
        if (hintCaretPosition === 0) {
            (_a = mainInputRef.current) === null || _a === void 0 ? void 0 : _a.focus();
            return;
        }
        if (!!hint && hint !== '') {
            setAvailableHint();
            setTimeout(function () {
                var _a, _b;
                (_a = mainInputRef.current) === null || _a === void 0 ? void 0 : _a.focus();
                var caretPosition = text.length + hintCaretPosition;
                (_b = mainInputRef.current) === null || _b === void 0 ? void 0 : _b.setSelectionRange(caretPosition, caretPosition);
            }, 0);
        }
    };
    var mainInput = react_1.cloneElement(child, __assign(__assign({}, childProps), { style: {
            boxSizing: 'border-box'
        }, onChange: onChange,
        onBlur: onBlur,
        onFocus: onFocus,
        onKeyDown: onKeyDown, ref: childProps.ref && typeof childProps.ref !== 'string'
            ? utils_1.mergeRefs(childProps.ref, mainInputRef)
            : mainInputRef }));
    return (react_1.default.createElement("div", { className: "rah-input-wrapper", style: {
            position: 'relative',
            display: 'inline-block'
        } }, disableHint
        ? child
        : (react_1.default.createElement(react_1.default.Fragment, null,
            mainInput,
            react_1.default.createElement("span", { className: "rah-hint-wrapper", ref: hintWrapperRef, style: {
                    display: 'flex',
                    pointerEvents: 'none',
                    backgroundColor: 'transparent',
                    borderColor: 'transparent',
                    boxSizing: 'border-box',
                    boxShadow: 'none',
                    color: 'rgba(0, 0, 0, 0.35)',
                    position: 'absolute',
                    top: 0,
                    left: 0,
                } },
                react_1.default.createElement("span", { style: {
                        visibility: 'hidden',
                        pointerEvents: 'none'
                    } }, text),
                react_1.default.createElement("input", { className: "rah-hint", ref: hintRef, onClick: onHintClick, style: {
                        pointerEvents: !hint || hint === '' ? 'none' : 'visible',
                        background: 'transparent',
                        width: '100%',
                        outline: 'none',
                        border: 'none',
                        boxShadow: 'none',
                        padding: 0,
                        margin: 0,
                        color: 'rgba(0, 0, 0, 0.35)',
                        caretColor: 'transparent'
                    }, value: hint, onChange: function () { return null; }, tabIndex: -1 }))))));
};
